@@ -49,7 +49,8 @@ typedef enum
   MPEG2DEC_FORMAT_NONE,
   MPEG2DEC_FORMAT_I422,
   MPEG2DEC_FORMAT_I420,
-  MPEG2DEC_FORMAT_YV12
+  MPEG2DEC_FORMAT_YV12,
+  MPEG2DEC_FORMAT_Y444
 } Mpeg2decFormat;
 
 typedef enum 
